import re

from django.test import TestCase
from django.core.mail import EmailMessage

from django_filtered_mail.mail import filter_recipients, filter_message


class StagingMailTestCase(TestCase):

    def test_filter(self):
        allowed = (
            re.compile(r'^.*@dreamsolution.nl$'),
            re.compile(r'^test-dreamsolution@gmail.com$'),
        )

        recipients = (
            'test@dreamsolution.nl',
            'test-dreamsolution@gmail.com',
            'example@example.com',
            'Some one <some-one@dreamsolution.nl>',
            'dreamsolution.nl@example.com',
        )

        recipients = filter_recipients(recipients, allowed, 'UTF-8')
        self.assertEqual(recipients, [
            'test@dreamsolution.nl',
            'test-dreamsolution@gmail.com',
            'Some one <some-one@dreamsolution.nl>',
        ])

    def test_message(self):
        allowed = [re.compile(r'^.*@dreamsolution.nl$')]

        # test valid address
        email = EmailMessage('Subject', 'Content', 'bounce@example.com',
            ['to@dreamsolution.nl'])
        email = filter_message(email, allowed)
        self.assertEqual(email.to, ['to@dreamsolution.nl'])

        # test invalid address
        email = EmailMessage('Subject', 'Content', 'bounce@example.com',
            ['test@gmail.com'])
        email = filter_message(email, allowed)
        self.assertEqual(email.to, [])

        # test one valid and one invalid 'To' address
        email = EmailMessage('Subject', 'Content', 'bounce@example.com',
            ['test@gmail.com', 'to@dreamsolution.nl'])
        email = filter_message(email, allowed)
        self.assertEqual(email.to, ['to@dreamsolution.nl'])

        # test multiple valid 'To' addresses
        email = EmailMessage('Subject', 'Content', 'bounce@example.com',
            ['test@dreamsolution.nl', 'to@dreamsolution.nl'])
        email = filter_message(email, allowed)
        self.assertEqual(email.to, ['test@dreamsolution.nl',
            'to@dreamsolution.nl'])

        # test 'Cc'
        email = EmailMessage('Subject', 'Content', 'bounce@example.com',
            ['test@gmail.com'], cc=['cc@dreamsolution.nl'])
        email = filter_message(email, allowed)
        self.assertEqual(email.cc, ['cc@dreamsolution.nl'])
        self.assertEqual(email.recipients(), ['cc@dreamsolution.nl'])

        # test 'Bcc'
        email = EmailMessage('Subject', 'Content', 'bounce@example.com',
            ['test@gmail.com', 'to@dreamsolution.nl'],
            bcc=['bcc@dreamsolution.nl'])
        email = filter_message(email, allowed)
        self.assertEqual(email.bcc, ['bcc@dreamsolution.nl'])
        self.assertEqual(email.recipients(), [
            'to@dreamsolution.nl', 'bcc@dreamsolution.nl'])

    # Testing the backend is harder, one needs a (local) SMTP server
    #def test_backend(self):
    #    connection = EmailBackend()
    #    connection.set_allowed_recipients([r'^.*@dreamsolution.nl$'])



""" Django email backend which behaves just like the default SMTP backend
except that it only sends messages to a select set of addresses.

See README.md for instructions
"""
import re
import logging
from email.utils import parseaddr

from django.core.mail.backends.smtp import EmailBackend as SMTPBackend
from django.core.mail.message import sanitize_address
from django.conf import settings


logger = logging.getLogger(__name__)


assert hasattr(settings, 'EMAIL_ALLOWED_RECIPIENTS'), (
    'Django settings is missing the EMAIL_ALLOWED_RECIPIENTS setting.')


def filter_recipients(recipients, patterns, encoding):
    """Filter out recipients not allowed in 'staging' environments
    Only looks at email addresses and not the 'name' part of recipients.
    """
    filtered = []
    for recip in recipients:
        recip = sanitize_address(recip, encoding)
        nm, addr = parseaddr(recip.lower())
        for pattern in patterns:
            if pattern.match(addr) is not None:
                filtered.append(recip)
                break
        else:
            logger.debug(u'Removed {0} from EmailMessage'.format(recip))

    return filtered


def filter_message(email_message, patterns):
    """Filter out recipients in to, cc and bcc of EmailMessage
    """
    for attr in ('to', 'cc', 'bcc'):
        recipients = getattr(email_message, attr, [])
        if recipients:
            recipients = filter_recipients(recipients, patterns,
                                           email_message.encoding)
            setattr(email_message, attr, recipients)
    return email_message


class EmailBackend(SMTPBackend):
    """ Django email backend which behaves just like the default SMTP backend
    except that it only sends messages to a filtered list of addresses.
    """
    def set_allowed_recipients(self, patterns):
        self._allowed_recipients = []
        for recip in patterns:
            self._allowed_recipients.append(re.compile(recip))

    def _send(self, email_message):
        if not getattr(self, '_allowed_recipients', None):
            self.set_allowed_recipients(settings.EMAIL_ALLOWED_RECIPIENTS)

        email_message = filter_message(email_message, self._allowed_recipients)
        return super(EmailBackend, self)._send(email_message)



from setuptools import setup, find_packages

import os

def get_long_description():
    path = os.path.join(os.path.dirname(__file__), 'README.md')
    with open(path) as f:
        return f.read()

setup(
    name='django-filtered-mail',
    version="0.2",
    packages=find_packages(),
    author='Rudolph Froger',
    author_email='rudolphfroger@dreamsolution.nl',
    license='MIT',
    description='Django Email Backend which only sends to whitelisted recipients',
    long_description=get_long_description(),
    url='https://github.com/rudolphfroger/django-filtered-mail',
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'django>=1.6',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Topic :: Communications :: Email :: Filters',
    ],
)
